@@ -332,7 +332,6 @@ int WINAPI vpb_call_sync(int handle, char *dialstr)
 	CALL	*c;
 
 	USHORT  b,ch;           // board and channel for this handle
-	VPBREG  *pvpbreg;
 #ifdef _OPENPRI
 	int     ch_state=0;
 	OpenPri *PriDsp;
@@ -348,8 +347,9 @@ int WINAPI vpb_call_sync(int handle, char *dialstr)
 
 		// Check if it is an ISDN channel
 		maphndletodev(handle, &b, &ch);
-		pvpbreg = vpb_c->vpbreg(b);
 #ifdef _OPENPRI
+		VPBREG  *pvpbreg = vpb_c->vpbreg(b);
+
 		if (pvpbreg->model == VPB_PRI){
 			mprintf("Got Pri card, checking channel state\n");
 			PriDsp=(OpenPri *)pvpbreg->hostdsp;
@@ -410,7 +410,6 @@ int WINAPI vpb_call_async(int handle, char *dialstr)
 	CALL	*c;
 
 	USHORT  b,ch;           // board and channel for this handle
-	VPBREG  *pvpbreg;
 #ifdef _OPENPRI
 	int     ch_state=0;
 	OpenPri *PriDsp;
@@ -425,8 +424,9 @@ int WINAPI vpb_call_async(int handle, char *dialstr)
 		strcpy(c->dialstr, dialstr);
 		// Check if it is an ISDN channel
 		maphndletodev(handle, &b, &ch);
-		pvpbreg = vpb_c->vpbreg(b);
 #ifdef _OPENPRI
+		VPBREG  *pvpbreg = vpb_c->vpbreg(b);
+
 		if (pvpbreg->model == VPB_PRI){
 			mprintf("Got Pri card, checking channel state\n");
 			PriDsp=(OpenPri *)pvpbreg->hostdsp;
@@ -938,14 +938,13 @@ void dial_isdn(void *data)
 	char    cid[21]="";
 	VPB_EVENT	e;
 	int 	ret, run = 1;
-	int 	state=IDLE;
 
 
 	mprintf("call:call_isdn making a call!\n");
 //	vpbdial_dial_async(c->handle, c->dialstr);
 	vpb_isdn_call(c->handle,c->dialstr,cid,0,0);
 //	c->state = WAIT_FOR_RINGBACK;
-	state = WAIT_FOR_RINGBACK;
+
 	while (run){
 		ret = vpb_get_event_ch_sync(c->handle,&e,SLEEPMS);
 		if (ret == VPB_OK){
