@@ -151,7 +151,7 @@ void WINAPI vpb_log_to_file(VPBPortHandle h, const std::string &filename)
 
 	pthread_mutex_unlock(&LogThreads[h].mutex);
 
-	if( pthread_create( &LogThreads[h].thread, NULL, logging_thread, (void*)h ) ) {
+	if( pthread_create( &LogThreads[h].thread, NULL, logging_thread, (void*)(intptr_t)h ) ) {
 		LogThreads[h].active = false;
 		throw VpbException("Failed to start logging thread for handle %d", h);
 	}
