@@ -398,7 +398,7 @@ static void extract_presentation_layer_message(
 { //{{{
 	int             msgtype=0;
 	int             bits_out[NBASCII];
-	int             start, i, j, blocks;
+	int             start, i, j;
 	int             valid_bits, last_bit;
 	int             state, valid;
 	unsigned char   ascii_char;
@@ -409,7 +409,6 @@ static void extract_presentation_layer_message(
 
 	// CSS/MSS processing ---------------------------------------------
 
-	blocks = n/N - 2;
 	start = 1;
 	last_bit = 0;
 	valid_bits = 0;
