@@ -635,7 +635,8 @@ int extract_jp_pres( char 	*pres,	// char array for bytes
 	int	error = 0;
 	int	state;
 	int	i = 0, j, k;
-	int	tos, tos_l = 0;
+//	int	tos;
+	int	tos_l = 0;
 	char	crc_l, crc_h;
 	unsigned long	cksum;
 	int	dle_trail=0;
@@ -836,7 +837,7 @@ int extract_jp_pres( char 	*pres,	// char array for bytes
 			}
 			else
 			{
-				tos= (int) ascii_char;
+				// tos= (int) ascii_char;
 				//printf("tos=%2d\n",tos);
 			}
 
