@@ -325,6 +325,19 @@ enum VPB_PORT
     //! Set the hook state of a port. Waits for it to change before returning.
     void WINAPI vpb_sethook_sync(VPBPortHandle handle, HookState hookstate);
 
+    //! Set the window for detecting flash events on OpenPCI cards.
+    //{{{
+    //! For FXS ports, this is automatically configured according to the locale
+    //! specific values in the @c Country structure.  For FXO ports, flash events
+    //! are only generated if this function has been explicitly called (this is
+    //! to maintain compatibility with older applications which manually processed
+    //! the raw on/off hook events themselves).
+    //!
+    //! @param handle The handle of the port to configure.
+    //! @param min    The minimum on-hook time for a valid flash signal in @c ms.
+    //! @param max    The maximum on-hook time for a valid flash signal in @c ms.
+    //}}}
+    void WINAPI vpb_set_flashtime(VPBPortHandle handle, uint16_t min, uint16_t max);
 
     // vpb_ring_station_async symbolic cadences
     #define VPB_RING_STATION_ON  -1 //!< Ring station port with default cadence
@@ -452,6 +465,7 @@ VT_DEPRECATED( int WINAPI vpb_conf(int h1,int resource, int mode) );
 	VPB_LOOP_ONHOOK		= 15,
 	VPB_LOOP_POLARITY	= 16,
 	VPB_DTMF_DOWN		= 17,
+	VPB_LOOP_FLASH		= 18,
 
 	// solicited (non maskable) events
 	VPB_PLAYEND		= 100,
@@ -509,6 +523,7 @@ VT_DEPRECATED( int WINAPI vpb_conf(int h1,int resource, int mode) );
     #define VPB_MLOOP_OFFHOOK		(1<<VPB_LOOP_OFFHOOK)
     #define VPB_MLOOP_ONHOOK		(1<<VPB_LOOP_ONHOOK)
     #define VPB_MLOOP_POLARITY		(1<<VPB_LOOP_POLARITY)
+    #define VPB_MLOOP_FLASH		(1<<VPB_LOOP_FLASH)
     #define VPB_MDTMF_DOWN		(1<<VPB_DTMF_DOWN)
     #define VPB_MALL_EVENTS	        ((unsigned long)-1)
 
