@@ -5,8 +5,8 @@
 	        Ron Lee, 8 Jun 2006
 
 
-	Copyright (C) 2005, 2006 Voicetronix www.voicetronix.com.au
-	Copyright (C) 2006, 2007 Ron Lee <ron@voicetronix.com.au>
+	Copyright (C) 2005 - 2013, Voicetronix www.voicetronix.com.au
+	Copyright (C) 2006 - 2013, Ron Lee <ron@voicetronix.com.au>
 
 	This library is free software; you can redistribute it and/or
 	modify it under the terms of the GNU General Public License
@@ -50,6 +50,7 @@
 #include <linux/cdev.h>
 #include <linux/device.h>
 #include <linux/proc_fs.h>
+#include <linux/seq_file.h>
 #include <linux/sched.h>
 #include <asm/uaccess.h>
 
@@ -124,6 +125,20 @@ MODULE_PARM_DESC(debug, "Set the logging verbosity");
 
 #endif
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(3,9,0)
+static inline void *PDE_DATA(const struct inode *inode)
+{
+	return PROC_I(inode)->pde->data;
+}
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(3,8,0)
+static inline struct inode *file_inode(struct file *f)
+{
+	return f->f_path.dentry->d_inode;
+}
+#endif
+
 
 /* Convenience macros for logging */
 #define boardinfo(board,format,...) info("[%d] " format, (board)->boardnum , ## __VA_ARGS__)
@@ -199,32 +214,41 @@ static inline const char *porttype(int type)
 } //}}}
 
 
-int vt_proc_read_int(char *page, char **start, off_t off,
-		     int count, int *eof, void *data)
+static int vt_int_proc_show(struct seq_file *m, void *v)
 { //{{{
-	/* We assume off < count here, and that our caller has sanity checked
-	 * this before we get here, if not, expect Bad Things to follow */
-	if( off > 0 ) return off;
-	else {
-	    int len = scnprintf(page, count, "%ld", (long)data);
-	    if( len > 0 ) *eof = 1;
-	    return len;
-	}
-} //}}}
+	seq_printf(m, "%ld", (long)m->private);
+	return 0;
+}
 
-int vt_proc_read_string(char *page, char **start, off_t off,
-			int count, int *eof, void *data)
+static int vt_int_proc_open(struct inode *inode, struct file *file)
+{
+	return single_open(file, vt_int_proc_show, PDE_DATA(inode));
+}
+
+const struct file_operations vt_int_proc_fops = {
+	.open		= vt_int_proc_open,
+	.read		= seq_read,
+	.llseek		= seq_lseek,
+	.release	= single_release,
+}; //}}}
+
+static int vt_string_proc_show(struct seq_file *m, void *v)
 { //{{{
-	/* We assume off < count here, and that our caller has sanity checked
-	 * this before we get here, if not, expect Bad Things to follow */
-	int datalen = strlen(data);
-	if( off >= datalen ) return off;
-	else {
-	    int len = scnprintf(page + off, count - off, "%s", ((char*)data) + off);
-	    if( len == datalen ) *eof = 1;
-	    return len;
-	}
-} //}}}
+	seq_puts(m, (const char*)m->private);
+	return 0;
+}
+
+static int vt_string_proc_open(struct inode *inode, struct file *file)
+{
+	return single_open(file, vt_string_proc_show, PDE_DATA(inode));
+}
+
+const struct file_operations vt_string_proc_fops = {
+	.open		= vt_string_proc_open,
+	.read		= seq_read,
+	.llseek		= seq_lseek,
+	.release	= single_release,
+}; //}}}
 
 
 int __init vtcore_init(void)
@@ -310,15 +334,8 @@ int __init vtcore_init(void)
 
 	vtcore->procfs_root = proc_mkdir("vt", NULL);
 	if( vtcore->procfs_root ) {
-		struct proc_dir_entry *p;
-
-		set_proc_owner(vtcore->procfs_root, THIS_MODULE);
-		p = create_proc_read_entry("maxboards", 0444,
-					   vtcore->procfs_root,
-					   vt_proc_read_int, (void*)MAX_BOARDS);
-		if (p) {
-			set_proc_owner(p, THIS_MODULE);
-		} else
+		if (!proc_create_data("maxboards", S_IRUGO, vtcore->procfs_root,
+					&vt_int_proc_fops, (void*)MAX_BOARDS))
 			crit("failed to create procfs maxboards");
 	} else
 		crit("failed to create procfs root");
@@ -1002,133 +1019,183 @@ static long vtcore_ioctl(struct file *file, unsigned int cmd, unsigned long data
 } //}}}
 
 
-int proc_read_country(char *page, char **start, off_t off, int count, int *eof, void *data)
+static int vt_country_proc_show(struct seq_file *m, void *v)
 { //{{{
 	struct vtboard *board;
+	unsigned	boardnum = (long)m->private >> 16;
+	unsigned	portnum  = (long)m->private & 0xff;
 
 	mutex_lock(&vtcore_mutex);
-	board = vtcore->boards[ (long)data >> 16 ];
-	if( off > 0 ) goto done;
-	if( try_module_get( board->owner ) ){
-		int len = scnprintf(page, count, "%s",
-				    board->chans[(long)data & 0xff].country);
-		if( len > 0 ) *eof = 1;
-		module_put( board->owner );
-		mutex_unlock(&vtcore_mutex);
-		dbgport(4,board,(int)(long)data & 0xff,"proc_read_country '%s'", page);
-		return len;
-	} else *eof = 1;
+	board = vtcore->boards[boardnum];
+
+	if (try_module_get(board->owner)) {
+		seq_puts(m, board->chans[portnum].country);
+		module_put(board->owner);
+	}
 
-    done:
 	mutex_unlock(&vtcore_mutex);
-	return off;
-} //}}}
+	return 0;
+}
 
-int proc_write_country(struct file *file, const char *buffer, unsigned long count, void *data)
-{ //{{{
-	struct vtboard *board;
-	char    buf[16] = { 0 };
-	char   *b = buf;
-	int     ret;
+static int vt_country_proc_open(struct inode *inode, struct file *file)
+{
+	return single_open(file, vt_country_proc_show, PDE_DATA(inode));
+}
 
-	if( count >= sizeof(buf) )               return -EINVAL;
-	if( copy_from_user(buf, buffer, count) ) return -EFAULT;
+static ssize_t vt_country_proc_write(struct file *file, const char *buffer,
+						size_t count, loff_t *pos)
+{
+	struct vtboard	*board;
+	void		*data		= PDE_DATA(file_inode(file));
+	unsigned	boardnum	= (long)data >> 16;
+	unsigned	portnum		= (long)data & 0xff;
+	char    	buf[16]		= { 0 };
+	char   		*b;
+	int     	ret;
+
+	if (count >= sizeof(buf))
+		return -EINVAL;
+
+	if (copy_from_user(buf, buffer, count))
+		return -EFAULT;
+
+	for (b = buf; *b != '\0'; ++b) {
+		if (*b == '\n') {
+			*b = '\0';
+			break;
+		}
+	}
 
-	while( *b != '\0' ) if(*b == '\n'){ *b = '\0'; break; } else ++b;
 	mutex_lock(&vtcore_mutex);
-	board = vtcore->boards[ (long)data >> 16 ];
-	if( try_module_get( board->owner ) ){
-		ret = board->set_country(board, (long)data & 0xff, buf);
-		if( ret == 0 ) ret = count;
-		module_put( board->owner );
-	} else  ret = -ENODEV;
+	board = vtcore->boards[boardnum];
+
+	if (try_module_get(board->owner)) {
+		ret = board->set_country(board, portnum, buf);
+		if (ret == 0)
+			ret = count;
+		module_put(board->owner);
+	} else
+		ret = -ENODEV;
+
 	mutex_unlock(&vtcore_mutex);
-	dbgport(4,board,(int)(long)data & 0xff,"proc_write_country '%s'", buf);
 	return ret;
-} //}}}
+}
+
+static const struct file_operations vt_country_proc_fops = {
+	.owner		= THIS_MODULE,
+	.open		= vt_country_proc_open,
+	.read		= seq_read,
+	.llseek		= seq_lseek,
+	.release	= single_release,
+	.write		= vt_country_proc_write,
+}; //}}}
 
 // Template definitions for port ops that communicate a single integer value.
 // {{{
-#define PROC_READ_PORT(attrib)						    \
-int proc_read_##attrib(char *page, char **start, off_t off, int count,	    \
-		       int *eof, void *data)				    \
-{									    \
-	struct vtboard *board;						    \
-									    \
-	mutex_lock(&vtcore_mutex);					    \
-	board = vtcore->boards[ (long)data >> 16 ];			    \
-	if( off > 0 ) goto done;					    \
-	if( try_module_get( board->owner ) ){				    \
-		int err;						    \
-		int ret = board->get_##attrib(board, (long)data & 0xff, &err);\
-		int len = err ? scnprintf(page, count, "ERR: %d", err)	    \
-			      : scnprintf(page, count, "%d", ret);	    \
-		if( len > 0 ) *eof = 1;					    \
-		module_put( board->owner );				    \
-		mutex_unlock(&vtcore_mutex);				    \
-		dbgport(4,board,(int)(long)data & 0xff,"proc_read_"#attrib " '%s'", page);\
-		return len;						    \
-	} else *eof = 1;						    \
-									    \
-    done:								    \
-	mutex_unlock(&vtcore_mutex);					    \
-	return off;							    \
+#define PROC_READ_PORT_(attrib)							\
+static int vt_##attrib##_proc_show(struct seq_file *m, void *v)			\
+{										\
+	struct vtboard *board;							\
+	unsigned	boardnum = (long)m->private >> 16;			\
+	unsigned	portnum  = (long)m->private & 0xff;			\
+										\
+	mutex_lock(&vtcore_mutex);						\
+	board = vtcore->boards[boardnum];					\
+										\
+	if (try_module_get(board->owner)) {					\
+		int err;							\
+		int ret = board->get_##attrib(board, portnum, &err);		\
+		if (err)							\
+			seq_printf(m, "ERR: %d", err);				\
+		else								\
+			seq_printf(m, "%d", ret);				\
+		module_put(board->owner);					\
+	}									\
+	mutex_unlock(&vtcore_mutex);						\
+	return 0;								\
+}										\
+										\
+static int vt_##attrib##_proc_open(struct inode *inode, struct file *file)	\
+{										\
+	return single_open(file, vt_##attrib##_proc_show, PDE_DATA(inode));	\
 }
 
-#define PROC_WRITE_PORT(attrib)						    \
-int proc_write_##attrib(struct file *file, const char *buffer,		    \
-			unsigned long count, void *data)		    \
-{									    \
-	struct vtboard *board;						    \
-	long    val;							    \
-	char    buf[8] = { 0 };						    \
-	char   *end;							    \
-	int     ret;							    \
-									    \
-	if( count >= sizeof(buf) )               return -EINVAL;	    \
-	if( copy_from_user(buf, buffer, count) ) return -EFAULT;	    \
-	val = simple_strtol(buf,&end,0);				    \
-	if( buf == end )                         return -EINVAL;	    \
-									    \
-	mutex_lock(&vtcore_mutex);					    \
-	board = vtcore->boards[ (long)data >> 16 ];			    \
-	if( try_module_get( board->owner ) ){				    \
-		ret = board->set_##attrib(board, (long)data & 0xff, val);   \
-		if( ret == 0 ) ret = count;				    \
-		module_put( board->owner );				    \
-	} else  ret = -ENODEV;						    \
-	mutex_unlock(&vtcore_mutex);					    \
-	dbgport(4,board,(int)(long)data & 0xff,"proc_write_"#attrib" '%s'", buf); \
-	return ret;							    \
+#define PROC_WRITE_PORT_(attrib)						\
+static ssize_t vt_##attrib##_proc_write(struct file *file, const char *buffer,	\
+						size_t count, loff_t *pos)	\
+{										\
+	struct vtboard *board;							\
+	void		*data		= PDE_DATA(file_inode(file));		\
+	unsigned	boardnum	= (long)data >> 16;			\
+	unsigned	portnum		= (long)data & 0xff;			\
+	long		val;							\
+	char		buf[8] = { 0 };						\
+	char		*end;							\
+	int		ret;							\
+										\
+	if (count >= sizeof(buf))						\
+		return -EINVAL;							\
+										\
+	if (copy_from_user(buf, buffer, count))					\
+		return -EFAULT;							\
+										\
+	val = simple_strtol(buf, &end, 0);					\
+	if (buf == end)								\
+		return -EINVAL;							\
+										\
+	mutex_lock(&vtcore_mutex);						\
+	board = vtcore->boards[boardnum];					\
+										\
+	if (try_module_get(board->owner)) {					\
+		ret = board->set_##attrib(board, portnum, val);			\
+		if (ret == 0)							\
+			ret = count;						\
+		module_put(board->owner);					\
+	} else									\
+		ret = -ENODEV;							\
+										\
+	mutex_unlock(&vtcore_mutex);						\
+	return ret;								\
 }
+
+#define PROC_READ_PORT(attrib)							\
+	PROC_READ_PORT_(attrib)							\
+										\
+static const struct file_operations vt_##attrib##_proc_fops = {			\
+	.open		= vt_##attrib##_proc_open,				\
+	.read		= seq_read,						\
+	.llseek		= seq_lseek,						\
+	.release	= single_release,					\
+};
+
+#define PROC_READWRITE_PORT(attrib)						\
+	PROC_READ_PORT_(attrib)							\
+	PROC_WRITE_PORT_(attrib)						\
+										\
+static const struct file_operations vt_##attrib##_proc_fops = {			\
+	.owner		= THIS_MODULE,						\
+	.open		= vt_##attrib##_proc_open,				\
+	.read		= seq_read,						\
+	.llseek		= seq_lseek,						\
+	.release	= single_release,					\
+	.write		= vt_##attrib##_proc_write,				\
+};
 //}}}
 
-PROC_READ_PORT(playgain)
-PROC_READ_PORT(recgain)
-PROC_READ_PORT(hook)
-PROC_READ_PORT(ring)
-PROC_READ_PORT(polarity)
+PROC_READWRITE_PORT(playgain)
+PROC_READWRITE_PORT(recgain)
+PROC_READWRITE_PORT(hook)
+PROC_READWRITE_PORT(ring)
+PROC_READWRITE_PORT(polarity)
 PROC_READ_PORT(linevolt)
-PROC_WRITE_PORT(playgain)
-PROC_WRITE_PORT(recgain)
-PROC_WRITE_PORT(hook)
-PROC_WRITE_PORT(ring)
-PROC_WRITE_PORT(polarity)
 
 // Create a read only proc entry 'node' for 'board',
 // returning the constant integer 'val'.
-int vt_create_board_proc_const_int(struct vtboard *board, const char *node, long val)
+void vt_create_board_proc_const_int(struct vtboard *board, const char *node, long val)
 { //{{{
-	struct proc_dir_entry *p;
-
-	p = create_proc_read_entry(node, 0444, board->procfs_root,
-					 vt_proc_read_int, (void*)val);
-	if( p ){
-		set_proc_owner(p, board->owner);
-		return RET_OK;
-	}
-	crit("failed to create procfs board%d/%s", board->boardnum, node);
-	return RET_FAIL;
+	if (!proc_create_data(node, S_IRUGO, board->procfs_root,
+				 &vt_int_proc_fops, (void*)val))
+		crit("failed to create procfs board%d/%s", board->boardnum, node);
 } //}}}
 
 
@@ -1321,31 +1388,18 @@ int vt_board_register(struct vtboard *board)
 		snprintf(dirname, sizeof(dirname)-1, "board%d", i);
 		board->procfs_root = proc_mkdir(dirname, vtcore->procfs_root);
 		if( board->procfs_root ) {
-			struct proc_dir_entry *p;
+			if (!proc_create_data("name", S_IRUGO, board->procfs_root,
+					&vt_string_proc_fops, (void*)board->name))
+				crit("failed to create procfs board%d/name", i);
 
-			set_proc_owner(board->procfs_root, board->owner);
-
-			p = create_proc_read_entry("name", 0444,
-						   board->procfs_root,
-						   vt_proc_read_string,
-						   (void*)board->name);
-			if (p) {
-				set_proc_owner(p, board->owner);
-			} else
-				crit("failed to create procfs board%d/maxports", i);
-
-			p = create_proc_read_entry("id", 0444,
-						   board->procfs_root,
-						   vt_proc_read_string,
-						   board->serial);
-			if (p) {
-				set_proc_owner(p, board->owner);
-			} else
+			if (!proc_create_data("id", S_IRUGO, board->procfs_root,
+					&vt_string_proc_fops, board->serial))
 				crit("failed to create procfs board%d/id", i);
 
 			vt_create_board_proc_const_int(board,"maxports",
 						       board->maxports);
-		} else crit("failed to create procfs board%d entry", board->boardnum);
+		} else
+			crit("failed to create procfs board%d entry", board->boardnum);
 	} //}}}
 
 	for(i=0; i < board->maxports; ++i)
@@ -1412,54 +1466,40 @@ int vt_board_register(struct vtboard *board)
 			snprintf(dirname, sizeof(dirname)-1, "port%d", i);
 			chan->procfs_root = proc_mkdir(dirname, board->procfs_root);
 			if( chan->procfs_root ) {
-				struct proc_dir_entry *p;
-				int    mode = 0;
+				int mode;
 
-				set_proc_owner(chan->procfs_root, board->owner);
-
-				p = create_proc_read_entry("type", 0444,
-							   chan->procfs_root,
-							   vt_proc_read_string,
-						    (void*)porttype(chan->porttype));
-				if (p) {
-					set_proc_owner(p, board->owner);
-				} else
+				if (!proc_create_data("type", S_IRUGO, chan->procfs_root,
+							&vt_string_proc_fops,
+							(void*)porttype(chan->porttype))) {
 					crit("failed to create board%d/port%d/type",
 								board->boardnum, i);
+				}
 
-				mode  = board->set_country ? 0200 : 0;
-				mode |= chan->country ? 0444 : 0;
-				if( mode ){ //{{{
-				    p = create_proc_entry("country",
-							  board->set_country ? 0644 : 0444,
-							  chan->procfs_root);
-				    if (p) {
-					    set_proc_owner(p, board->owner);
-					    p->read_proc  = proc_read_country;
-					    p->write_proc = proc_write_country;
-					    p->data = (void*)(((long)board->boardnum << 16) | i);
-				    } else
-					    crit("failed to create board%d/port%d/country",
+				if (board->set_country || chan->country) {
+					if (!proc_create_data("country",
+						S_IRUGO | (board->set_country ? S_IWUSR : 0),
+						chan->procfs_root,
+						&vt_country_proc_fops,
+						(void*)(((long)board->boardnum << 16) | i))) {
+
+						crit("failed to create board%d/port%d/country",
 									board->boardnum, i);
-				} //}}}
-
+					}
+				}
 
 		// Template for a port op passing a single int value.
-		#define DEFINE_PROC_PORT_OP_(attrib,mode)    			    \
-			if( mode ){				    		    \
-			    p = create_proc_entry(#attrib, mode,    		    \
-						  chan->procfs_root);	    	    \
-			    if (p) {						    \
-				    set_proc_owner(p, board->owner);		    \
-				    p->read_proc  = proc_read_##attrib;		    \
-				    p->write_proc = proc_write_##attrib;	    \
-				    p->data = (void*)(((long)board->boardnum << 16) | i); \
-			    } else  crit("failed to create board%d/port%d/"#attrib, \
-					 board->boardnum, i);			    \
+		#define DEFINE_PROC_PORT_OP_(attrib,mode)				\
+			if (mode) {							\
+				if (!proc_create_data(#attrib, mode, chan->procfs_root,	\
+							&vt_##attrib##_proc_fops,	\
+					(void*)(((long)board->boardnum << 16) | i))) {	\
+					crit("failed to create board%d/port%d/"#attrib, \
+								board->boardnum, i);    \
+				}							\
 			}
-		#define DEFINE_PROC_PORT_OP(attrib)	    			    \
-			mode  = board->set_##attrib ? 0200 : 0;	    		    \
-			mode |= board->get_##attrib ? 0444 : 0;		    	    \
+		#define DEFINE_PROC_PORT_OP(attrib)			\
+			mode  = board->set_##attrib ? S_IWUSR : 0;	\
+			mode |= board->get_##attrib ? S_IRUGO : 0;	\
 			DEFINE_PROC_PORT_OP_(attrib,mode)
 
 				DEFINE_PROC_PORT_OP(playgain)
@@ -1468,23 +1508,14 @@ int vt_board_register(struct vtboard *board)
 				switch( chan->porttype ){
 				    case VT_FXO_AN:
 					DEFINE_PROC_PORT_OP(hook)
-					DEFINE_PROC_PORT_OP_(ring,0444)
-					if( board->get_linevolt ) {
-						p = create_proc_read_entry("linevolt", 0444,
-									   chan->procfs_root,
-									   proc_read_linevolt,
-						    (void*)(((long)board->boardnum << 16) | i));
-						if (p) {
-							set_proc_owner(p, board->owner);
-						} else
-							crit("failed to create board%d/"
-									"port%d/linevolt",
-									board->boardnum, i);
-					}
+					DEFINE_PROC_PORT_OP_(ring, S_IRUGO)
+
+					if (board->get_linevolt)
+						DEFINE_PROC_PORT_OP_(linevolt, S_IRUGO)
 					break;
 
 				    case VT_FXS_AN:
-					DEFINE_PROC_PORT_OP_(hook,0444)
+					DEFINE_PROC_PORT_OP_(hook, S_IRUGO)
 					DEFINE_PROC_PORT_OP(ring)
 					DEFINE_PROC_PORT_OP(polarity)
 				}
@@ -1564,6 +1595,11 @@ void vt_board_unregister(struct vtboard *board)
 	if( board->procfs_root ){
 		char  dirname[8] = { 0 };
 
+		// We could just replace all of this (and the above lot) with:
+		// proc_remove(board->procfs_root);
+		// but it and remove_proc_subtree didn't appear until v3.9-rc5
+		// so stick with what works with older kernels for a bit longer.
+
 		snprintf(dirname, sizeof(dirname)-1, "board%d", board->boardnum);
 		remove_proc_entry("name", board->procfs_root);
 		remove_proc_entry("id", board->procfs_root);
@@ -1587,14 +1623,8 @@ void vt_echo_register(struct vtecho *ec)
 	mutex_lock(&vtcore_mutex);
 	vtcore->echo_can = ec;
 	if( vtcore->procfs_root ) {
-		struct proc_dir_entry *p = create_proc_read_entry("echo_canceller",
-								  0444,
-								  vtcore->procfs_root,
-								  vt_proc_read_string,
-								  ec->desc);
-		if (p) {
-			set_proc_owner(p, THIS_MODULE);
-		} else
+		if (!proc_create_data("echo_canceller", S_IRUGO, vtcore->procfs_root,
+						&vt_string_proc_fops, ec->desc))
 			crit("failed to create procfs echo_canceller");
 	}
 	info("echo canceller [%s] registered",ec->desc);
@@ -2185,7 +2215,6 @@ void fifo_memcpy(char *dest, char *src, int length)
 }
 
 
-/* Export Symbols - functions to be used by other kernel modules */
 EXPORT_SYMBOL(vt_board_register);
 EXPORT_SYMBOL(vt_board_unregister);
 EXPORT_SYMBOL(vt_echo_register);
@@ -2193,8 +2222,8 @@ EXPORT_SYMBOL(vt_echo_unregister);
 EXPORT_SYMBOL(vt_write);
 EXPORT_SYMBOL(vt_read);
 EXPORT_SYMBOL(vt_send_event);
-EXPORT_SYMBOL(vt_proc_read_int);
-EXPORT_SYMBOL(vt_proc_read_string);
+EXPORT_SYMBOL(vt_int_proc_fops);
+EXPORT_SYMBOL(vt_string_proc_fops);
 EXPORT_SYMBOL(vt_create_board_proc_const_int);
 
 module_init(vtcore_init);
