@@ -2044,7 +2044,6 @@ static irqreturn_t openpci_isr(int irq, void *dev_id)
 	/* Check what interrupted us */
 	spin_lock_irqsave(&card->lock, flags);
 	status = inb(TJ_INTSTAT);
-	outb(status, TJ_INTSTAT);
 
 	/* No TJ status flags?  Check for an ARM interrupt */
 	if( unlikely( ! status ) ) {
@@ -2092,6 +2091,12 @@ static irqreturn_t openpci_isr(int irq, void *dev_id)
                 return IRQ_HANDLED;
             #endif
 	}
+
+	if (unlikely((status & 0x03) == 0))
+		status |= status >> 2 & 0x03;
+
+	outb(status, TJ_INTSTAT);
+
 	if(unlikely(status & 0x10)) {
 		/* To recover the PCI master state machine set the reset bit
 		   in Reg0 to 1 and reset it to 0, also enable DMA operation
