@@ -104,8 +104,9 @@ int main(int argc, char * argv[])
 }
 
 void *port_thread(void *pv) {
-	int       h = *(int*)pv;
-	VPB_EVENT e;
+	int       	h = *(int*)pv;
+	VPB_EVENT 	e;
+	char		s[VPB_MAX_STR];
 
 	pthread_mutex_lock(&mutex);
 	threads_active++;
@@ -116,16 +117,14 @@ void *port_thread(void *pv) {
 	while(!finito) {
 
 		if( vpb_get_event_ch_sync(h, &e, 1000) != VPB_OK ) continue;
-
-		if ((e.type == VPB_RING) || (e.type == VPB_STATION_OFFHOOK)) {
+		vpb_translate_event(&e,s);
+		printf("Event Type[%d] chan %s", e.type,s);
+		if ((e.type == VPB_RING) /*|| (e.type == VPB_STATION_OFFHOOK)*/) {
 			printf("[%02d] answering.....\n",h);
-			vpb_sethook_sync(h,VPB_OFFHOOK);
-
+			vpb_sethook_async(h,VPB_OFFHOOK);
 			printf("[%02d] sleeping for 10 sec ...\n",h);
-//			ev = ctwaitforevent(h, VPB_DROP, 10000);
 			vpb_sleep(10000);
-
-			vpb_sethook_sync(h, VPB_ONHOOK);
+			vpb_sethook_async(h, VPB_ONHOOK);
 			printf("[%02d] done.....\n",h);
 		}
 	}
